# -*- coding: utf-8 -*-

"""
Usage example::

    import acore_server_monitoring_core.api as acore_server_monitoring_core
"""

from .dynamodb import make_dynamodb_table_name
from .dynamodb import UseCaseEnum
from .dynamodb import Measurement
from .dynamodb import WorldServerStatusMeasurement
from .dynamodb import Ec2RdsStatusMeasurement



# -*- coding: utf-8 -*-

import enum
import pynamodb_mate.api as pm


def make_dynamodb_table_name(env_name: str) -> str:  # pragma: no cover
    return f"wserver-{env_name}-server-monitoring"


class UseCaseEnum(str, enum.Enum):
    worldserver_status = "worldserver_status"
    ec2_rds_status = "ec2_rds_status"


class Measurement(pm.Model):
    """
    Acore server monitoring time series measurement data store.

    :param series_id: the time series id, usually it is in format of
        ``{server_id}-{use_case_id}``, for example ``sbx-blue-worldserver_status``,
        ``sbx-blue-ec2_rds_status``.
    :param create_at: the time when the measurement data is created.
    :param expire_at: the time when the measurement data is expired. DynamoDB
        will automatically delete the expired data in a few days.
    """

    series_id: pm.REQUIRED_STR = pm.UnicodeAttribute(hash_key=True)
    create_at: pm.REQUIRED_DATETIME = pm.UTCDateTimeAttribute(range_key=True)
    expire_at: pm.REQUIRED_DATETIME = pm.TTLAttribute()


class WorldServerStatusMeasurement(Measurement):
    """
    World server status measurement data.

    :param is_ec2_exists: is EC2 exists.
    :param is_rds_exists: is RDS exists.
    :param is_ec2_running: is EC2 running.
    :param is_rds_running: is RDS running.
    :param ec2_status: EC2 status in string.
    :param rds_status: RDS status in string.
    :param connected_players: connected players.
    :param characters_in_world: characters in world.
    :param server_uptime: server uptime in seconds.
    :param cpu_usage: 0 ~ 100, float. 50.0 means 50%.
    :param memory_usage: 0 ~ 100, float. 50.0 means 50%.
    :param total_memory: total memory in MB.
    :param available_memory: available memory in MB.
    """

    is_ec2_exists: pm.OPTIONAL_BOOL = pm.BooleanAttribute(default=None, null=True)
    is_rds_exists: pm.OPTIONAL_BOOL = pm.BooleanAttribute(default=None, null=True)
    is_ec2_running: pm.OPTIONAL_BOOL = pm.BooleanAttribute(default=None, null=True)
    is_rds_running: pm.OPTIONAL_BOOL = pm.BooleanAttribute(default=None, null=True)
    ec2_status: pm.OPTIONAL_STR = pm.UnicodeAttribute(default=None, null=True)
    rds_status: pm.OPTIONAL_STR = pm.UnicodeAttribute(default=None, null=True)
    connected_players: pm.OPTIONAL_INT = pm.NumberAttribute(default=None, null=True)
    characters_in_world: pm.OPTIONAL_INT = pm.NumberAttribute(default=None, null=True)
    server_uptime: pm.OPTIONAL_INT = pm.NumberAttribute(default=None, null=True)
    cpu_usage: pm.OPTIONAL_FLOAT = pm.NumberAttribute(default=None, null=True)
    memory_usage: pm.OPTIONAL_FLOAT = pm.NumberAttribute(default=None, null=True)
    total_memory: pm.OPTIONAL_INT = pm.NumberAttribute(default=None, null=True)
    available_memory: pm.OPTIONAL_INT = pm.NumberAttribute(default=None, null=True)


class Ec2RdsStatusMeasurement(Measurement):
    """
    EC2 and RDS status measurement data.

    :param is_ec2_exists: is EC2 exists.
    :param is_rds_exists: is RDS exists.
    :param is_ec2_running: is EC2 running.
    :param is_rds_running: is RDS running.
    :param ec2_status: EC2 status in string.
    :param rds_status: RDS status in string.
    """

    is_ec2_exists: pm.OPTIONAL_BOOL = pm.BooleanAttribute(default=None, null=True)
    is_rds_exists: pm.OPTIONAL_BOOL = pm.BooleanAttribute(default=None, null=True)
    is_ec2_running: pm.OPTIONAL_BOOL = pm.BooleanAttribute(default=None, null=True)
    is_rds_running: pm.OPTIONAL_BOOL = pm.BooleanAttribute(default=None, null=True)
    ec2_status: pm.OPTIONAL_STR = pm.UnicodeAttribute(default=None, null=True)
    rds_status: pm.OPTIONAL_STR = pm.UnicodeAttribute(default=None, null=True)



# -*- coding: utf-8 -*-

from pathlib import Path

dir_here = Path(__file__).absolute().parent
PACKAGE_NAME = dir_here.name

dir_project_root = dir_here.parent

# ------------------------------------------------------------------------------
# Virtual Environment Related
# ------------------------------------------------------------------------------
dir_venv = dir_project_root / ".venv"
dir_venv_bin = dir_venv / "bin"

# virtualenv executable paths
bin_pytest = dir_venv_bin / "pytest"

# test related
dir_htmlcov = dir_project_root / "htmlcov"
path_cov_index_html = dir_htmlcov / "index.html"
dir_unit_test = dir_project_root / "tests"



# -*- coding: utf-8 -*-

from acore_server_monitoring_core import api


def test():
    _ = api
    _ = api.make_dynamodb_table_name
    _ = api.UseCaseEnum
    _ = api.Measurement
    _ = api.WorldServerStatusMeasurement
    _ = api.Ec2RdsStatusMeasurement


if __name__ == "__main__":
    from acore_server_monitoring_core.tests import run_cov_test

    run_cov_test(__file__, "acore_server_monitoring_core.api", preview=False)



# -*- coding: utf-8 -*-

import os
import sys
import contextlib
import subprocess
from pathlib import Path


@contextlib.contextmanager
def temp_cwd(path: Path):
    """
    Temporarily set the current working directory (CWD) and automatically
    switch back when it's done.
    """
    cwd = os.getcwd()
    os.chdir(str(path))
    try:
        yield path
    finally:
        os.chdir(cwd)


def run_cov_test(
    script: str,
    module: str,
    root_dir: str,
    htmlcov_dir: str,
    preview: bool = False,
    is_folder: bool = False,
):
    """
    The pytest-cov plugin gives you the coverage for entire project. What if
    I want run per-module test independently and get per-module coverage?

    This is a simple wrapper around pytest + coverage cli command. Allow you to run
    coverage test from Python script and set the code coverage measurement scope.

    Usage example:

    suppose you have a source code folder structure like this::

        /dir_git_repo/
        /dir_git_repo/my_library
        /dir_git_repo/my_library/__init__.py
        /dir_git_repo/my_library/module1.py
        /dir_git_repo/my_library/module2.py

    In your module 1 unit test script, you can do this:

    .. code-block:: python

        from my_library.module1 import func1, func2

        def test_func1():
            pass

        def test_func2():
            pass

        if __name__ == "__main__":
            from fixa.pytest_cov_helper import run_cov_test

            run_cov_test(
                script=__file__,
                module="my_library.module1", # test scope is the module1.py
                root_dir="/path/to/dir_git_repo",
                htmlcov_dir="/path/to/dir_git_repo/htmlcov",
            )

    In your all modules unit test script, you can do this:

    .. code-block:: python

        if __name__ == "__main__":
            from fixa.pytest_cov_helper import run_cov_test

            run_cov_test(
                script=__file__,
                module="my_library", # test scope is the my_library/
                root_dir="/path/to/dir_git_repo",
                htmlcov_dir="/path/to/dir_git_repo/htmlcov",
                is_folder=True, # my_library is a folder
            )

    :param script: the path to test script
    :param module: the dot notation to the python module you want to calculate
        coverage
    :param root_dir: the dir to dump coverage results binary file
    :param htmlcov_dir: the dir to dump HTML output
    :param preview: whether to open the HTML output in web browser after the test
    :param is_folder: whether the module is a folder

    Reference:

    - https://pypi.org/project/pytest-cov/
    """
    bin_pytest = Path(sys.executable).parent / "pytest"
    if is_folder:
        script = f"{Path(script).parent}"
    if module.endswith(".py"):  # pragma: no cover
        module = module[:-3]
    args = [
        f"{bin_pytest}",
        "-s",
        "--tb=native",
        f"--rootdir={root_dir}",
        f"--cov={module}",
        "--cov-report",
        "term-missing",
        "--cov-report",
        f"html:{htmlcov_dir}",
        script,
    ]
    with temp_cwd(Path(root_dir)):
        subprocess.run(args)
    if preview:  # pragma: no cover
        platform = sys.platform
        if platform in ["win32", "cygwin"]:
            open_command = "start"
        elif platform in ["darwin", "linux"]:
            open_command = "open"
        else:
            raise NotImplementedError
        subprocess.run([open_command, f"{Path(htmlcov_dir).joinpath('index.html')}"])
